@@ -290,10 +290,10 @@ gst_mp3parse_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&mp3_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&mp3_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &mp3_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &mp3_src_template);
 
   GST_DEBUG_CATEGORY_INIT (mp3parse_debug, "mp3parse", 0, "MPEG Audio Parser");
 
@@ -321,10 +321,10 @@ gst_mp3parse_class_init (GstMPEGAudioParseClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SKIP,
       g_param_spec_int ("skip", "skip", "skip",
-          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BIT_RATE,
       g_param_spec_int ("bitrate", "Bitrate", "Bit Rate",
-          G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_mp3parse_change_state;
 
