@@ -188,8 +188,8 @@ gst_dvd_demux_base_init (gpointer klass)
   mpeg_parse_class->parse_packhead = gst_dvd_demux_parse_packhead;
 
   /* sink pad */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template);
 
   demux_class->audio_template = gst_static_pad_template_get (&audio_template);
 
