@@ -325,15 +325,15 @@ gst_rdt_manager_base_init (gpointer klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   /* sink pads */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rdt_manager_recv_rtp_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rdt_manager_recv_rtcp_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rdt_manager_recv_rtp_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rdt_manager_recv_rtcp_sink_template);
   /* src pads */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rdt_manager_recv_rtp_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rdt_manager_rtcp_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rdt_manager_recv_rtp_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rdt_manager_rtcp_src_template);
 
   gst_element_class_set_details_simple (element_class, "RTP Decoder",
       "Codec/Parser/Network",
